@@ -33,6 +33,7 @@
 /* === H E A D E R S ======================================================= */
 
 #include <cassert>
+#include <typeinfo>
 
 /* === M A C R O S ========================================================= */
 
@@ -73,6 +74,8 @@ private:
 
 protected:
 	shared_object():refcount(0) { }
+	shared_object(const shared_object&):refcount(0) { }
+	shared_object& operator= (const shared_object&) { return *this; }
 
 #ifdef ETL_SELF_DELETING_SHARED_OBJECT
 	virtual ~shared_object() { }
@@ -151,8 +154,10 @@ class virtual_shared_object
 {
 protected:
 	virtual_shared_object() { }
+	virtual_shared_object(const virtual_shared_object&) { }
+	virtual_shared_object& operator= (const virtual_shared_object&) { return *this; }
 public:
-	virtual ~virtual_shared_object()=0;
+	virtual ~virtual_shared_object() { }
 	virtual void ref()const=0;
 	virtual bool unref()const=0;
 	virtual bool unref_inactive()const=0;
@@ -338,7 +343,19 @@ public:
 	//! static_cast<> overload -- Useful for implicit casts
 	template <class U>
 	operator handle<U>()const
-	{ return handle<U>(static_cast<U*>(obj)); }
+	{ return handle<U>(obj); }
+
+	template<typename U>
+	bool type_is() const
+	{ return dynamic_cast<const U*>(obj); }
+
+	template<typename U>
+	U* type_pointer() const
+	{ return dynamic_cast<U*>(obj); }
+
+	template<typename U>
+	bool type_equal() const
+	{ return typeid(*obj) == typeid(U); }
 }; // END of template class handle
 
 // ========================================================================
@@ -358,6 +375,8 @@ public:
 
 protected:
 	rshared_object():rrefcount(0),front_(0),back_(0) { }
+	rshared_object(const rshared_object &other): shared_object(other), rrefcount(0),front_(0),back_(0) { }
+	rshared_object& operator= (const rshared_object&) { return *this; }
 
 public:
 	virtual void rref()const
@@ -695,7 +714,7 @@ public:
 		if(x.get()==obj)
 			return *this;
 
-		obj=static_cast<value_type*>(x.get());
+		obj=x.get();
 		return *this;
 	}
 
@@ -705,7 +724,7 @@ public:
 		if(x.get()==obj)
 			return *this;
 
-		obj=static_cast<value_type*>(x.get());
+		obj=x.get();
 		return *this;
 	}
 
@@ -785,6 +804,18 @@ public:
 	void ref() { if(obj)obj->ref(); }
 
 	bool unref() { if(obj && !obj->unref()){ obj=0; return false; } return true; }
+
+	template<typename U>
+	bool type_is() const
+	{ return dynamic_cast<const U*>(obj); }
+
+	template<typename U>
+	U* type_pointer() const
+	{ return dynamic_cast<U*>(obj); }
+
+	template<typename U>
+	bool type_equal() const
+	{ return typeid(*obj) == typeid(U); }
 }; // END of template class loose_handle
 
 // cast loose_handle<> -> handle<>
